#!/bin/bash

ELF_FILENAME="simple_socket_server_plus.elf"
SOF_FILENAME="nios2_linux_3c120_125mhz_top.sof"
CABLE="1"
DEVICE="1"
NIOS2_INSTANCE="0"
SIDP="0x8004d40"
ID="482137343u"
TIMESTAMP="1241815090u"
FLASH_BASE_ADDRESS="0x00000000"
FLASH_END_ADDRESS="0x03FFFFFF"
HW_CONFIG_OFFSET="0x03500000"
NIOS2_RESET_ADDRESS="0x02800000"
BOOT_COPIER_SREC="${SOPC_KIT_NIOS2}/components/altera_nios2/boot_loader_sources/boot_loader_cfi.srec"
PFL_OPTION_BITS_FILE="./pfl_option_bits/option_bits.flash"

#
# Verify that the input files specified above actually exist...
#

if [ ! -f ${SOF_FILENAME} ] ; then
	echo ""
	echo "ERROR: the speified SOF file does not appear to exist..."
	echo "SOF file: ${SOF_FILENAME}"
	echo ""

	exit 1
fi

if [ ! -f ${ELF_FILENAME} ] ; then
	echo ""
	echo "ERROR: the speified ELF file does not appear to exist..."
	echo "ELF file: ${ELF_FILENAME}"
	echo ""

	exit 1
fi

if [ ! -f ${PFL_OPTION_BITS_FILE} ] ; then
	echo ""
	echo "ERROR: the speified PFL option bits file does not appear to exist..."
	echo "PFL file: ${PFL_OPTION_BITS_FILE}"
	echo ""

	exit 1
fi

echo ""
echo "This script attempts to program the CFI flash device with the FPGA"
echo "configuration data and the Nios II software application."
echo ""
USER_INPUT="NO"
while [ ${USER_INPUT} != "YES" ] ; do

	echo -n "Do you wish to proceed? (Y/N):"
	read USER_INPUT
	case ${USER_INPUT} in
		[Yy])
			echo "Continuing the script..."
			USER_INPUT="YES"
			;;
		[Nn])
			echo "Exiting the script..."
			exit 1
			;;
		*)
			USER_INPUT="NO"
			;;
	esac
done

CMD="nios2-download --cable ${CABLE} --device ${DEVICE} --instance ${NIOS2_INSTANCE} --sidp ${SIDP} --id ${ID} --timestamp ${TIMESTAMP} --reset-target"
echo ""
echo "#"
echo "# Attempting to determine if the FPGA is already configured with this command:"
echo "#"
echo ${CMD}
echo ""

${CMD} || {
	CMD="nios2-configure-sof --cable ${CABLE} --device ${DEVICE} ${SOF_FILENAME}"
	echo ""
	echo "#"
	echo "# Attempting to configure the FPGA with this command:"
	echo "#"
	echo ${CMD}
	echo ""
	
	${CMD} || {
		echo ""
		echo "#"
		echo "# ERROR: while attempting to configure the FPGA..."
		echo "#"
		echo ""
	
		exit 1
	}

	CMD="nios2-download --cable ${CABLE} --device ${DEVICE} --instance ${NIOS2_INSTANCE} --sidp ${SIDP} --id ${ID} --timestamp ${TIMESTAMP} --reset-target"
	echo ""
	echo "#"
	echo "# Attempting to determine if the FPGA is configured properly with this command:"
	echo "#"
	echo ${CMD}
	echo ""

	${CMD} || {
		echo ""
		echo "#"
		echo "# ERROR: cannot verify FPGA contains correct confiuration..."
		echo "#"
		echo ""
	
		exit 1
	}
}

SOF_FLASH_FILENAME="${SOF_FILENAME}.flash"
CMD="sof2flash --offset=${HW_CONFIG_OFFSET} --input=${SOF_FILENAME} --output=${SOF_FLASH_FILENAME}"
echo ""
echo "#"
echo "# Attempting to create the SOF flash file with this command:"
echo "#"
echo ${CMD}
echo ""

${CMD} || {
	echo ""
	echo "#"
	echo "# ERROR: While creating SOF flash file..."
	echo "#"
	echo ""

	exit 1
}

ELF_FLASH_FILENAME="${ELF_FILENAME}.flash"
CMD="elf2flash --base=${FLASH_BASE_ADDRESS} --end=${FLASH_END_ADDRESS} --reset=${NIOS2_RESET_ADDRESS} --boot=${BOOT_COPIER_SREC} --input=${ELF_FILENAME} --output=${ELF_FLASH_FILENAME}"
echo ""
echo "#"
echo "# Attempting to create the ELF flash file with this command:"
echo "#"
echo ${CMD}
echo ""

${CMD} || {
	echo ""
	echo "#"
	echo "# ERROR: While creating ELF flash file..."
	echo "#"
	echo ""

	exit 1
}

CMD="nios2-flash-programmer --cable ${CABLE} --device ${DEVICE} --instance ${NIOS2_INSTANCE} --sidp ${SIDP} --id ${ID} --timestamp ${TIMESTAMP} --base ${FLASH_BASE_ADDRESS} ${SOF_FLASH_FILENAME}"
echo ""
echo "#"
echo "# Attempting to program the SOF image into CFI flash with this command:"
echo "#"
echo ${CMD}
echo ""

${CMD} || {
	echo ""
	echo "#"
	echo "# ERROR: While programming SOF image into CFI flash..."
	echo "#"
	echo ""

	exit 1
}

CMD="nios2-flash-programmer --cable ${CABLE} --device ${DEVICE} --instance ${NIOS2_INSTANCE} --sidp ${SIDP} --id ${ID} --timestamp ${TIMESTAMP} --base ${FLASH_BASE_ADDRESS} ${ELF_FLASH_FILENAME}"
echo ""
echo "#"
echo "# Attempting to program the ELF image into CFI flash with this command:"
echo "#"
echo ${CMD}
echo ""

${CMD} || {
	echo ""
	echo "#"
	echo "# ERROR: While programming ELF image into CFI flash..."
	echo "#"
	echo ""

	exit 1
}

CMD="nios2-flash-programmer --cable ${CABLE} --device ${DEVICE} --instance ${NIOS2_INSTANCE} --sidp ${SIDP} --id ${ID} --timestamp ${TIMESTAMP} --base ${FLASH_BASE_ADDRESS} ${PFL_OPTION_BITS_FILE}"
echo ""
echo "#"
echo "# Attempting to program the PFL option bits into CFI flash with this command:"
echo "#"
echo ${CMD}
echo ""

${CMD} || {
	echo ""
	echo "#"
	echo "# ERROR: While programming PFL option bits into CFI flash..."
	echo "#"
	echo ""

	exit 1
}

echo ""
echo "#"
echo "# CFI flash should now be programmed..."
echo "#"
echo "# Ensure that the \"PGM CONFIG SELECT\" rotary switch is in position 0 and then"
echo "# press the \"RESET_CONFIG\" push button to reconfigure your board from flash."
echo "#"
echo ""

exit 0
