#!/bin/bash

SOF_FILENAME="nios2_linux_3c120_125mhz_top.sof"
CABLE="1"
DEVICE="1"
NIOS2_INSTANCE="0"
SIDP="0x8004d40"
ID="482137343u"
TIMESTAMP="1241815090u"
FLASH_BASE_ADDRESS="0x00000000"

#
# Verify that the input files specified above actually exist...
#

if [ ! -f ${SOF_FILENAME} ] ; then
	echo ""
	echo "ERROR: the speified SOF file does not appear to exist..."
	echo "SOF file: ${SOF_FILENAME}"
	echo ""

	exit 1
fi

echo ""
echo "This script attempts to completely erase the contents of the CFI flash device."
echo ""
USER_INPUT="NO"
while [ ${USER_INPUT} != "YES" ] ; do

	echo -n "Do you wish to proceed? (Y/N):"
	read USER_INPUT
	case ${USER_INPUT} in
		[Yy])
			echo "Continuing the script..."
			USER_INPUT="YES"
			;;
		[Nn])
			echo "Exiting the script..."
			exit 1
			;;
		*)
			USER_INPUT="NO"
			;;
	esac
done

CMD="nios2-download --cable ${CABLE} --device ${DEVICE} --instance ${NIOS2_INSTANCE} --sidp ${SIDP} --id ${ID} --timestamp ${TIMESTAMP} --reset-target"
echo ""
echo "#"
echo "# Attempting to determine if the FPGA is already configured with this command:"
echo "#"
echo ${CMD}
echo ""

${CMD} || {
	CMD="nios2-configure-sof --cable ${CABLE} --device ${DEVICE} ${SOF_FILENAME}"
	echo ""
	echo "#"
	echo "# Attempting to configure the FPGA with this command:"
	echo "#"
	echo ${CMD}
	echo ""
	
	${CMD} || {
		echo ""
		echo "#"
		echo "# ERROR: while attempting to configure the FPGA..."
		echo "#"
		echo ""
	
		exit 1
	}

	CMD="nios2-download --cable ${CABLE} --device ${DEVICE} --instance ${NIOS2_INSTANCE} --sidp ${SIDP} --id ${ID} --timestamp ${TIMESTAMP} --reset-target"
	echo ""
	echo "#"
	echo "# Attempting to determine if the FPGA is configured properly with this command:"
	echo "#"
	echo ${CMD}
	echo ""

	${CMD} || {
		echo ""
		echo "#"
		echo "# ERROR: cannot verify FPGA contains correct confiuration..."
		echo "#"
		echo ""
	
		exit 1
	}
}

CMD="nios2-flash-programmer --cable ${CABLE} --device ${DEVICE} --instance ${NIOS2_INSTANCE} --sidp ${SIDP} --id ${ID} --timestamp ${TIMESTAMP} --base ${FLASH_BASE_ADDRESS} --erase-all"
echo ""
echo "#"
echo "# Attempting to erase entire CFI flash with this command:"
echo "#"
echo ${CMD}
echo ""

${CMD} || {
	echo ""
	echo "#"
	echo "# ERROR: While erasing CFI flash..."
	echo "#"
	echo ""

	exit 1
}

echo ""
echo "#"
echo "# CFI flash should now be completely erased..."
echo "#"
echo ""

exit 0
